#
#
# Copyright (c)2004 DuckCorp(tm)
#
#
#
# This file is part of Coin-diff.
#
# Coin-diff is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Coin-diff is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Coin-diff; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#



require 'coin-diff/pkgdeb'
require "rexml/document"

module CoinDiff

	PRODUCT = "Coin-diff"
	VERSION = "0.1.5"

	class Config
		@@default_conf_path = ["/etc/coin-diff/", "~/.coin-diff/"]
		@@conf_filename = "coin-diff.conf"
		@@available_path_list = ["data", "work", "web"]
		@@arch_list = ["alpha", "amd64", "arm", "hppa", "hurd-i386", "i386", "ia64", "m68k", "mips", "mipsel", "powerpc", "s390", "sh", "sparc", "kfreebsd-i386"]

		def initialize()
			clear
		end

		def clear()
			@conf_path = ""
			@path_list = {}
			@base_url = "/coin-diff/"
			@repo_list_src = {}
			@repo_list_bin = {}
		end

		def load(conf_path = nil)
			clear

			if conf_path
				conf_path = File.expand_path(conf_path)
				conf_path += "/" if conf_path[-1..-1] != "/"
				@conf_path = conf_path
			else
				@@default_conf_path.each do |path|
					path = File.expand_path(path)
					path += "/" if path[-1..-1] != "/"
					@conf_path = path if FileTest.exists?(path) and FileTest.directory?(path)
				end
			end
			if @conf_path == ""
				STDERR.puts "No config path specified and none of default ones are available !"
				return false
			end
			conffile = @conf_path + @@conf_filename
			begin
				file = File.new(conffile)
			rescue
				STDERR.puts "Problems fetching '"+conffile+"': "+$!
				return false
			end

			doc = REXML::Document.new(file)
			return false if not file
			root = doc.root;
			root.elements.each("path_list/path") do |el|
				path = File.expand_path(el.get_text.to_s)
				path += "/" if path[-1..-1] != "/"
				@path_list[el.attributes['use']] = path if @@available_path_list.include?(el.attributes['use'])
			end

			root.elements.each("repo_list/src_repo") do |el|
				@repo_list_src[el.elements['name'].get_text.to_s] = el.elements['location'].get_text.to_s
			end

			root.elements.each("repo_list/bin_repo") do |el|
				arch = el.attributes['arch']
				if not @@arch_list.include?(arch)
					STDERR.puts "Bad binary repository entry : unknown arch '"+arch+"' !"
					next
				end
				list = {}
				list = @repo_list_bin[arch] if @repo_list_bin.has_key?(arch)
				list[el.elements['name'].get_text.to_s] = el.elements['location'].get_text.to_s
				@repo_list_bin[arch] = list
			end
			
			@base_url = root.elements['web_output'].elements['base_url'].get_text.to_s
			@base_url = "/" + @base_url if @base_url[0..0] != "/" and @base_url[0..3] != "http"
			@base_url += "/" if @base_url[-1..-1] != "/"
			return true
		end

		def get_data_path()
			return @path_list["data"] if @path_list["data"]
			return "/usr/share/coin-diff/"
		end

		def get_work_path()
			return @path_list["work"] if @path_list["work"]
			return "/var/lib/coin-diff/"
		end

		def get_web_path()
			return @path_list["web"] if @path_list["web"]
			return "/var/www/coin-diff/"
		end

		def get_repo_list_src()
			return @repo_list_src
		end

		def get_arch_list()
			return @repo_list_bin.keys
		end

		def get_repo_list_bin(arch)
			return @repo_list_bin[arch] if @repo_list_bin.has_key?(arch)
			return {}
		end

		def setup_pkgbrowser(browser)
			if not browser.classtree.include?(CoinDiff::PkgBrowser)
				STDERR.puts "Cannot setup browser : not a browser !"
				return false
			end

			browser.feed_using_repolist_list(@repo_list_src, false)

			get_arch_list.each do |arch|
				browser.feed_using_repolist_list(@repo_list_bin[arch], true)

				filename = get_data_path + "pkg_ignore_list_" + arch
				browser.feed_arch_ignore_from_file(arch, filename) if FileTest.exists?(filename) and FileTest.readable?(filename)
			end

			return true
		end

		def get_base_url()
			return @base_url
		end

		def get_db_filename()
			return get_work_path() + "pkg_infos.db"
		end
	end

end

