#!/usr/bin/ruby
#
#
# Copyright (c)2004 DuckCorp(tm)
#
#
#
# This file is part of Coin-diff.
#
# Coin-diff is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Coin-diff is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Coin-diff; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#



require 'getoptlong'
require 'fileutils'
require 'coin-diff/coindiff'

opts = GetoptLong.new(
	[ "--config-path", "-c", GetoptLong::REQUIRED_ARGUMENT ]
)
config_path = nil
opts.each do |opt, arg|
	config_path = arg if opt == "--config-path"
end

config = CoinDiff::Config.new
config.load(config_path)

data_path = config.get_data_path
work_path = config.get_work_path
web_path = config.get_web_path

base_url = config.get_base_url
arch_default = "hurd-i386"
nb_pkg_cols = 4

browser = CoinDiff::PkgBrowser.new
browser.load_pkg_db(config.get_db_filename())

arch_list = browser.get_arch_list()
obs_list = browser.check_obsolescence().sort
lvl_list = CoinDiff::PkgBin.priority_lvl.sort { |a,b| b[1] <=> a[1] }
list_all = browser.get_pkgset_list()

render = HTML_RENDER.new
def pkgset_display_link(base_url, pkgset)
	return "<a href='"+base_url+"pkgset/"+pkgset.name+"/'>"+pkgset.name+"</a>"
end
def sourcepkgs_display_link(base_url, pkgset)
	return "<a href='"+base_url+"pkgset/"+pkgset.name+"/sources.html'>"+pkgset.name+" source packages</a>"
end
def binpkgs_display_link(base_url, pkgset)
	return "<a href='"+base_url+"pkgset/"+pkgset.name+"/binaries.html'>"+pkgset.name+" binaries packages</a>"
end
def print_timestamp(file)
	file.puts "<p>"
	file.puts "These pages are updated daily.<br />"
	file.puts "Last updated : <ins>"+Time.now.strftime("%Y-%m-%dT%H:%M:%S")+"</ins>"
	file.puts "</p>"
end
def begin_file(web_path, data_path, filename, title, backlink=nil)
	file = File.open(web_path+"/"+filename, "w")
	IO.foreach(data_path+"/pkg_list.header") {|line| file.puts line }
	file.puts "<h1>"+title+"</h1>"
	file.puts "<p><a href='"+backlink+"'>&lt;-- Back &lt;--</a></p>" if backlink
	print_timestamp(file)
	return file
end
def end_file(web_path, data_path, file)
	file.puts "<p>Generated by "+CoinDiff::PRODUCT+" V"+CoinDiff::VERSION+" .</p>"
	IO.foreach(data_path+"/pkg_list.footer") {|line| file.puts line }
	file.close
end



begin
	FileUtils::rm([web_path+"/pkg_list.html", web_path+"/pkg_list_all.html", web_path+"/pkg_list_missing.html", web_path+"/pkg_list_uncomplete.html", web_path+"/pkg_list_not_uptodate.html"]) 
rescue
end

arch_list.each do |arch|

	list_missing = browser.get_missing(arch).keys
	list_uncomplete = browser.get_uncomplete(arch).keys
	list_not_uptodate = browser.get_not_uptodate(arch, list_missing).keys
	sets_lists_all = browser.split_by_priority(browser.list_to_pkgsets(list_all), true)
	sets_lists_missing = browser.split_by_priority(browser.list_to_pkgsets(list_missing), true)
	sets_lists_uncomplete = browser.split_by_priority(browser.list_to_pkgsets(list_uncomplete), true)
	sets_lists_not_uptodate = browser.split_by_priority(browser.list_to_pkgsets(list_not_uptodate), true)

	ignore_list = browser.get_ignore_list(arch)
	ignore_list_match = browser.get_ignore_list_match(arch)
	ignore_list_bad = browser.get_ignore_list_bad(arch)

	repo_src = browser.get_src_repo_list()
	repo_bon = browser.get_bin_arch_repo_list(arch)

	nb_sources = browser.get_pkgset_count
	nb_ignore_list = ignore_list.length.to_s
	nb_ignored = ignore_list_match.length
	nb_bad = ignore_list_bad.length.to_s
	nb_kept_sources = nb_sources - nb_ignored
	nb_missing = list_missing.length
	nb_built = nb_kept_sources - nb_missing
	nb_uncomplete = list_uncomplete.length
	nb_fully_built = nb_built - nb_uncomplete
	nb_not_uptodate = list_not_uptodate.length
	nb_updated = nb_fully_built - nb_not_uptodate



	file = begin_file(web_path, data_path, "pkg_list_"+arch+".html", "Debian GNU/Hurd Packages information")
	FileUtils::ln_s("pkg_list_"+arch+".html", web_path+"/pkg_list.html") if (arch == arch_default)

	file.puts "<p>"
	file.puts "Source packages : "+nb_sources.to_s+"<br />"
	file.puts "Source packages ignored : "+nb_ignored.to_s+" ("+nb_bad.to_s+"/"+nb_ignore_list.to_s+" bad entries in <a href='#ignore_list'>ignore patterns</a>) (Linux specific or of no use on Hurd)<br />"
	file.puts "</p>"

	file.puts "<p>"
	file.puts "Source packages built : "+nb_built.to_s+" / "+nb_kept_sources.to_s+"<br />"
	file.puts "Source packages missing : "+nb_missing.to_s+" / "+nb_kept_sources.to_s+"<br />"
	file.puts "<br />"
	file.puts "Source packages fully built : "+nb_fully_built.to_s+" / "+nb_built.to_s+"<br />"
	file.puts "Source packages uncomplete : "+nb_uncomplete.to_s+" / "+nb_built.to_s+"<br />"
	file.puts "<br />"
	file.puts "Source packages updated : "+nb_updated.to_s+" / "+nb_fully_built.to_s+"<br />"
	file.puts "Source packages not-up-to-date : "+nb_not_uptodate.to_s+" / "+nb_fully_built.to_s+"<br />"
	file.puts "</p>"

	file.puts "<p>"
	file.puts "The following pages are work in progress and may contain errors.<br />"
	file.puts "</p>"

	file.puts "<h2>Links :</h2>"
	file.puts "<div>"
	file.puts "<ul>"
	file.puts "<li><a href='pkg_list_all.html'>All packages List</a></li>"
	file.puts "<li><a href='pkg_list_missing.html'>Missing packages List</a></li>"
	file.puts "<li><a href='pkg_list_uncomplete.html'>Uncomplete packages List</a></li>"
	file.puts "<li><a href='pkg_list_not_uptodate.html'>Not Up-To-Date packages List</a></li>"
	file.puts "</ul>"
	file.puts "</div>"

	file.puts "<div>"
	file.puts "<span><b>Source code :</b></span>"
	file.puts "<ul>"
	file.puts "<li><a href='"+base_url+"/CoinDiff.rb'>CoinDiff.rb</a> (classes used to manipulate Debian Packages information)</li>"
	file.puts "<li><a href='"+base_url+"/pkg_feeder.rb'>pkg_feeder.rb</a> (script to fetch pkg data, compute things, and feed the DB)</li>"
	file.puts "<li><a href='"+base_url+"/pkg_browser.rb'>pkg_browser.rb</a> (script reading the DB, gathering useful information to generate HTML-ized output)</li>"
	file.puts "</ul>"
	file.puts "</div>"

	file.puts render.display_simple_tab("Source Debian repositories", repo_src) { |title, url| title+" : "+url }
	file.puts render.display_simple_tab("Binary Debian repositories (hurd-i386)", repo_bon) { |title, url| title+" : "+url }

	file.puts "<div id='ignore_list'>"
	file.puts render.display_multicol_tab("List of ignore patterns", ignore_list, nb_pkg_cols) { |el| el }
	file.puts render.display_multicol_tab("List of ignored source packages", ignore_list_match, nb_pkg_cols) {|pkgset| "<a href='"+base_url+"pkgset/"+pkgset+"/'>"+pkgset+"</a>" }
	file.puts render.display_multicol_tab("List of bad / obsolete ignore patterns", ignore_list_bad, nb_pkg_cols) { |el| el } if ignore_list_bad.length > 0
	file.puts "</div>"

	file.puts "<div>"
	file.puts render.display_multicol_tab("List of obsolescent source packages", obs_list, nb_pkg_cols) {|pkgset| "<a href='"+base_url+"pkgset/"+pkgset+"/'>"+pkgset+"</a>" }
	file.puts "</div>"

	end_file(web_path, data_path, file)



	file = begin_file(web_path, data_path, "pkg_list_all_"+arch+".html", "All Pkg List", "pkg_list_"+arch+".html")
	FileUtils::ln_s("pkg_list_all_"+arch+".html", web_path+"/pkg_list_all.html") if (arch == arch_default)

	file.puts "<p>"
	file.puts "Source packages built : "+nb_built.to_s+" / "+nb_kept_sources.to_s+"<br />"
	file.puts "Source packages missing : "+nb_missing.to_s+" / "+nb_kept_sources.to_s+"<br />"
	file.puts "</p>"

	lvl_list.each do |lvl, lvl_prio|
		file.puts render.display_multicol_tab("List of all packages of priority '"+lvl+"'", sets_lists_all[lvl], nb_pkg_cols) { |pkgset| pkgset_display_link(base_url, pkgset) }
	end
	file.puts render.display_multicol_tab("List of all unclassable packages (not enought binary pkg to compute a source priority)", sets_lists_all[nil], nb_pkg_cols) { |pkgset| pkgset_display_link(base_url, pkgset) } if sets_lists_all[nil].length > 0

	end_file(web_path, data_path, file)



	file = begin_file(web_path, data_path, "pkg_list_missing_"+arch+".html", "Missing Pkg List", "pkg_list_"+arch+".html")
	FileUtils::ln_s("pkg_list_missing_"+arch+".html", web_path+"/pkg_list_missing.html") if (arch == arch_default)

	file.puts "<p>"
	file.puts "Source packages built : "+nb_built.to_s+" / "+nb_kept_sources.to_s+"<br />"
	file.puts "Source packages missing : "+nb_missing.to_s+" / "+nb_kept_sources.to_s+"<br />"
	file.puts "</p>"

	lvl_list.each do |lvl, lvl_prio|
		file.puts render.display_multicol_tab("List of missing packages of priority '"+lvl+"'", sets_lists_missing[lvl], nb_pkg_cols) {|pkgset| pkgset_display_link(base_url, pkgset) }
	end
	file.puts render.display_multicol_tab("List of unclassable missing packages (not enought binary pkg to compute a source priority)", sets_lists_missing[nil], nb_pkg_cols) {|pkgset| pkgset_display_link(base_url, pkgset) } if sets_lists_missing[nil].length > 0

	end_file(web_path, data_path, file)



	file = begin_file(web_path, data_path, "pkg_list_uncomplete_"+arch+".html", "Uncomplete Pkg List", "pkg_list_"+arch+".html")
	FileUtils::ln_s("pkg_list_uncomplete_"+arch+".html", web_path+"/pkg_list_uncomplete.html") if (arch == arch_default)

	file.puts "<p>"
	file.puts "Source packages fully built : "+nb_fully_built.to_s+" / "+nb_built.to_s+"<br />"
	file.puts "Source packages uncomplete : "+nb_uncomplete.to_s+" / "+nb_built.to_s+"<br />"
	file.puts "</p>"

	lvl_list.each do |lvl, lvl_prio|
		file.puts render.display_multicol_tab("List of uncomplete packages of priority '"+lvl+"'", sets_lists_uncomplete[lvl], nb_pkg_cols) { |pkgset| pkgset_display_link(base_url, pkgset) }
	end
	file.puts render.display_multicol_tab("List of unclassable uncomplete packages (not enought binary pkg to compute a source priority)", sets_lists_uncomplete[nil], nb_pkg_cols) { |pkgset| pkgset_display_link(base_url, pkgset) } if sets_lists_uncomplete[nil].length > 0

	end_file(web_path, data_path, file)



	file = begin_file(web_path, data_path, "pkg_list_not_uptodate_"+arch+".html", "Not-Up-To-Date Pkg List", "pkg_list_"+arch+".html")
	FileUtils::ln_s("pkg_list_not_uptodate_"+arch+".html", web_path+"/pkg_list_not_uptodate.html") if (arch == arch_default)

	file.puts "<p>"
	file.puts "Source packages updated : "+nb_updated.to_s+" / "+nb_fully_built.to_s+"<br />"
	file.puts "Source packages not-up-to-date : "+nb_not_uptodate.to_s+" / "+nb_fully_built.to_s+"<br />"
	file.puts "</p>"

	lvl_list.each do |lvl, lvl_prio|
		file.puts render.display_multicol_tab("List of not-up-to-date packages of priority '"+lvl+"'", sets_lists_not_uptodate[lvl], nb_pkg_cols) { |pkgset| pkgset_display_link(base_url, pkgset) }
	end
	file.puts render.display_multicol_tab("List of unclassable not-up-to-date packages (not enought binary pkg to compute a source priority)", sets_lists_not_uptodate[nil], nb_pkg_cols) { |pkgset| pkgset_display_link(base_url, pkgset) } if sets_lists_not_uptodate[nil].length > 0

	end_file(web_path, data_path, file)

end



FileUtils::rm_rf([web_path+"/pkgset"])
FileUtils::makedirs([web_path+"/pkgset"])
FileUtils::ln_s([data_path+"/default.css"], web_path+"/pkgset")

list_all.each do |pkgset|
	set = browser.get_pkgset(pkgset)
	FileUtils::makedirs([web_path+"/pkgset/"+pkgset.to_s])
	FileUtils::ln_s([web_path+"/default.css"], web_path+"/pkgset/"+pkgset.to_s)

	file = begin_file(web_path, data_path, "pkgset/"+pkgset.to_s+"/index.html", "Pkgset information", "../../pkg_list.html")
	file.puts "<h2>Links :</h2>"
	file.puts "<div>"
	file.puts "<ul>"
	file.puts "<li>"+sourcepkgs_display_link(base_url, set)+"</li>"
	file.puts "<li>"+binpkgs_display_link(base_url, set)+"</li>"
	file.puts "</ul>"
	file.puts "</div>"
	file.puts "<pre>"
	file.puts set.to_s
	file.puts "</pre>"
	end_file(web_path, data_path, file)

	file = begin_file(web_path, data_path, "pkgset/"+pkgset.to_s+"/sources.html", "Pkgset source packages", base_url+"/pkgset/"+pkgset.to_s+"/")
	src_ver_list = set.get_src_version_list.sort!
	src_ver_list.each do |ver|
		file.puts "<fieldset>"
		file.puts "<legend>"+pkgset+" ("+ver+")</legend>"
		file.puts "<pre>"
		file.puts set.get_src(ver).to_s
		file.puts "</pre>"
		file.puts "</fieldset>"
	end
	end_file(web_path, data_path, file)

	file = begin_file(web_path, data_path, "pkgset/"+pkgset.to_s+"/binaries.html", "Pkgset binary packages", base_url+"/pkgset/"+pkgset.to_s+"/")
	bin_ver_list = set.get_bin_version_list.sort!
	bin_ver_list.each do |ver|
		set.get_bin_list_for_ver(ver).each do |bin|
			if set.bin_archall?(bin)
					if set.bin_available?("all", bin, ver)
						file.puts "<fieldset>"
						file.puts "<legend>"+bin+" ("+ver+" / all)</legend>"
						file.puts "<pre>"
						file.puts set.get_bin("all", bin, ver).to_s
						file.puts "</pre>"
						file.puts "</fieldset>"
					end
			else
				set.get_arch_list.each do |arch|
					if set.bin_available?(arch, bin, ver)
						file.puts "<fieldset>"
						file.puts "<legend>"+bin+" ("+ver+" / "+arch+")</legend>"
						file.puts "<pre>"
						file.puts set.get_bin(arch, bin, ver).to_s
						file.puts "</pre>"
						file.puts "</fieldset>"
					end
				end
			end
		end
	end
	end_file(web_path, data_path, file)
end

