#!/usr/bin/ruby

require 'coin-diff/coindiff'

config = CoinDiff::Config.new
config.load()


module CoinDiff

	class PkgUI

		def initialize(data_path, web_path, base_url)
			@data_path = data_path
			@web_path = web_path
			@base_url = base_url
			@header = ""
			@content = ""
			@footer = ""
		end

		def display_header(stylesheet = nil)
			@header = ""
			@header += '<?xml version="1.0" encoding="utf-8" ?>' + "\n";
			@header += '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd" >' + "\n";
			@header += '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">' + "\n";
			@header += '<head>' + "\n";
			@header += '<title>' + CoinDiff::PRODUCT + '</title>' + "\n";
			@header += '<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />' + "\n";
			@header += '<meta http-equiv="Pragma" content="no-cache" />' + "\n";
			@header += '<link rel="stylesheet" type="text/css" href="' + @base_url + 'data/default.css" />' + "\n";
			if (stylesheet)
				@header += '<link rel="stylesheet" type="text/css" href="' + stylesheet + '" />' + "\n";
			end
			@header += '</head>' + "\n";
			@header += '<body>' + "\n";
		end

		def display_footer()
			@footer = ""
			@footer += '<div id="footer">' + "\n";
			@footer += '<p>Generated by ' + CoinDiff::PRODUCT + ' V' + CoinDiff::VERSION + '</p>' + "\n"
			@footer += '<a href="http://validator.w3.org/check?uri=referer"><img src="http://www.w3.org/Icons/valid-xhtml11" alt="Valid XHTML 1.1!" height="31" width="88" /></a>' + "\n";
			@footer += '</div>' + "\n";
			@footer += '</body>' + "\n";
			@footer += '</html>' + "\n";
		end

		def add_content(content)
			@content += content
		end

		def render(filename = nil)
			result = @header + @content + @footer
			if not filename
				puts @result
				return true
			else
				begin
					file = File.open(filename, "w")
					file.puts result
					file.close
					return true
				rescue
					STDERR.puts "Problems creating '" + filename + "': " + $!
				end
			end
			return false
		end

		def gen_title(title, type, subtitle="")
			res = ""
			case type
				when "main_page"
					res += '<h1 class="mtitle_before">' + title + '</h1>' + "\n" if title != "" and subtitle != ""
					res += '<h1 class="mtitle_left">' + title + '</h1>' + "\n" if title != "" and subtitle == ""
					res += '<h1 class="mtitle_right">' + subtitle + '</h1>' + "\n" if subtitle != ""
			end
			return res
		end

		def gen_msgbox(title, type, content)
			res = ""
			case type
				when "moalabox"
					res += '<div class="moalabox">'
					res += '<h2 class="moalabox">' + title + '</h2>'
					res += content
					res += '</div>'
			end
			return res
		end

	end

end


browser = CoinDiff::PkgBrowser.new
#browser.load_pkg_db(config.get_db_filename())
browser.load_pkg_db("/tmp/pkg_list.db")

# zlib  e2fsprogs  sed

set = browser.get_pkgset("sed")

ui = CoinDiff::PkgUI.new(config.get_data_path(), config.get_web_path(), config.get_base_url())
ui.display_header()
ui.add_content('<div id="content">' + "\n")
ui.add_content(ui.gen_title("UI test", "main_page", "zZz"))
ui.add_content(ui.gen_title("UI test", "main_page", ""))
ui.add_content(ui.gen_title("", "main_page", "zZz"))
ui.add_content(ui.gen_msgbox("box test", "moalabox", "zzzzzzzzzzz<br />123<br />456"))
#ui.display_src_pkg(src)
ui.add_content('</div>' + "\n")
ui.display_footer()
ui.render("/var/www/coin-diff/test.html")

